@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, Thomas Jaeger <ThJaeger@gmail.com>
+ * Copyright (c) 2008-2009, Thomas Jaeger <ThJaeger@gmail.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,9 +18,9 @@
 #include "prefdb.h"
 #include <string>
 #include <map>
-#include <X11/Xlib.h>
 #include <X11/extensions/XInput.h>
-#include <X11/cursorfont.h>
+
+#define MAX_BUTTONS 256
 
 class XAtom {
 	const char *name;
@@ -46,69 +46,120 @@ class Grabber;
 extern Grabber *grabber;
 
 float rescaleValuatorAxis(int coord, int fmin, int fmax, int tmax);
+bool has_wm_state(Window w);
+bool has_atom(Window w, Atom prop, Atom value);
+
+void queue(sigc::slot<void> f);
+std::string select_window();
+void fake_core_button(guint b, bool press);
 
 class Grabber {
+	friend class Handler;
+	friend class StrokeHandler;
+	friend class Button;
 public:
 	Children children;
-	enum State { NONE, BUTTON, ALL_SYNC, ALL_ASYNC, SCROLL, SELECT };
-	static const char *state_name[6];
+	enum State { NONE, BUTTON, ALL_SYNC, SELECT };
+	static const char *state_name[4];
 	enum EventType { DOWN = 0, UP = 1, MOTION = 2, BUTTON_MOTION = 3, PROX_IN = 4, PROX_OUT = 5 };
 	bool xinput;
 	bool proximity_selected;
-	bool is_event(int, EventType);
-	unsigned int get_device_button_state();
+	int event_type[6];
+	bool is_event(int type, EventType et) { return xinput && type == event_type[et]; }
 	void select_proximity();
 
 	struct XiDevice {
+		std::string name;
+		std::map<guint, guint> inv_map;
 		XDevice *dev;
 		XEventClass events[6];
-		int event_type[6];
 		int all_events_n;
 		bool supports_proximity, supports_pressure;
+		bool active;
 		int pressure_min, pressure_max;
 		int min_x, max_x, min_y, max_y;
+		bool absolute;
+		int valuators[2];
+		int num_buttons;
 		int normalize_pressure(int pressure) {
 			return 255 * (pressure - pressure_min) / (pressure_max - pressure_min);
 		}
+		void fake_button(int b, bool press);
+		void release_all();
+		void grab_device(bool grab);
+		void grab_button(ButtonInfo &bi, bool grab);
+		void update_pointer_mapping();
+		void translate_coords(int *axis_data, float &x, float &y);
+		bool translate_known_coords(int sx, int sy, int *axis_data, float &x, float &y);
 	};
+
+	unsigned int get_device_button_state(XiDevice *&dev);
 	XiDevice *get_xi_dev(XID id);
+	int mapping_notify;
 	int event_presence;
 	XEventClass presence_class;
-private:
-	int button_events_n;
+
 	XiDevice **xi_devs;
 	int xi_devs_n;
+	int nMajor;
+private:
 	bool init_xi();
 
 	State current, grabbed;
 	bool xi_grabbed;
-	bool suspended;
+	bool xi_devs_grabbed;
+	int suspended;
 	bool active;
-	Cursor cursor_scroll, cursor_select;
-	std::map<guint, guint> buttons;
+	bool is_disabled;
+	Cursor cursor_select;
+	ButtonInfo grabbed_button;
+	std::vector<ButtonInfo> buttons;
 	bool timing_workaround;
 
 	void set();
 	void grab_xi(bool);
+	void regrab_xi();
+	void grab_xi_devs(bool);
+	std::string get_wm_class(Window w);
+	std::string wm_class;
+
+	void update_excluded();
+
+	void grab(State s) { current = s; set(); }
+	void suspend() { suspended++; set(); }
+	void resume() { if (suspended) suspended--; set(); }
+	void update();
 public:
 	Grabber();
 	~Grabber();
 	bool handle(XEvent &ev) { return children.handle(ev); }
-	std::string get_wm_class(Window w);
-	void update(Window w) { active = !prefs.exceptions.ref().count(get_wm_class(w)); set(); }
+	std::string get_wm_class() { return wm_class; }
 
-	void get_button();
-	void fake_button(int b);
-	void grab(State s) { current = s; set(); }
-	void suspend() { suspended = true; set(); }
-	void resume() { suspended = false; set(); }
-	void regrab() { suspend(); grab_xi(false); get_button(); resume(); grab_xi(true); }
-	void grab_xi_devs(bool);
-	bool is_grabbed(guint b) { return buttons.find(b) != buttons.end(); }
+	void queue_suspend() { queue(sigc::mem_fun(*this, &Grabber::suspend)); }
+	void queue_resume() { queue(sigc::mem_fun(*this, &Grabber::resume)); }
+
+	bool update_device_list();
+
+	bool is_grabbed(guint b);
+	bool is_instant(guint b);
+	bool is_click_hold(guint b);
+	void release_all(int n = 0);
+
+	int get_default_button() { return grabbed_button.button; }
+	bool get_timing_workaround() { return timing_workaround; }
+
+	void unminimize();
 };
 
+extern Grabber::XiDevice *current_dev;
+extern std::set<guint> xinput_pressed;
+
 class GrabFailedException : public std::exception {
-	virtual const char* what() const throw() { return "Grab Failed"; }
+	char *msg;
+public:
+	GrabFailedException(int code) { if (asprintf(&msg, "Grab Failed: %d", code) == -1) msg = NULL; }
+	virtual const char* what() const throw() { return msg ? msg : "Grab Failed"; }
+	~GrabFailedException() throw() { free(msg); }
 };
 
 #endif
