@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, Thomas Jaeger <ThJaeger@gmail.com>
+ * Copyright (c) 2008-2009, Thomas Jaeger <ThJaeger@gmail.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,8 +19,11 @@
 #include <glibmm.h>
 
 class Timeout {
+	// Invariant: c == &connection || c == NULL
 	sigc::connection *c;
-	bool to() { timeout(); c = 0; return false; }
+	sigc::connection connection;
+	// We have to account for the possibilty that timeout() destroys the object
+	bool to() { c = NULL; timeout(); return false; }
 public:
 	Timeout() : c(0) {}
 protected:
@@ -29,7 +32,6 @@ public:
 	bool remove_timeout() {
 		if (c) {
 			c->disconnect();
-			delete c;
 			c = 0;
 			return true;
 		}
@@ -37,7 +39,8 @@ public:
 	}
 	void set_timeout(int ms) {
 		remove_timeout();
-		c = new sigc::connection(Glib::signal_timeout().connect(sigc::mem_fun(*this, &Timeout::to), ms));
+		connection = Glib::signal_timeout().connect(sigc::mem_fun(*this, &Timeout::to), ms);
+		c = &connection;
 	}
 	virtual ~Timeout() {
 		remove_timeout();
